# -- BEGIN LICENSE BLOCK ----------------------------------------------

# catmux
# Copyright (C) 2018  Felix Mauch
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# -- END LICENSE BLOCK ------------------------------------------------

"""Contains everything around the config file"""
import re
import yaml

from catmux.window import Window
import catmux.tmux_wrapper as tmux
import catmux.exceptions as cme


def check_boolean_field(boolean):
    if isinstance(boolean, bool):
        return boolean
    return boolean.lower() in ("yes", "true", "t", "1", True)


class Session(object):

    """Parser for a config yaml file"""

    def __init__(self, server_name, session_name, runtime_params=None):
        """TODO: to be defined1."""

        self._server_name = server_name
        self._session_name = session_name
        self._parameters = dict()
        self._runtime_params = self._parse_overwrites(runtime_params)
        self._windows = list()
        self.__yaml_data = None

        self._before_commands = list()
        self._default_window = None

    def init_from_filepath(self, filepath):
        """Initializes the data from a file read from filepath."""

        try:
            self.__yaml_data = yaml.safe_load(open(filepath, "r"))
        except yaml.YAMLError as exc:
            raise cme.InvalidConfig(
                f"Config file {filepath} is not a valid yaml file:\n{exc}"
            )

        self.init_from_yaml(self.__yaml_data)

    def init_from_yaml(self, yaml_data):
        """Initialize config directly by an already loaded yaml structure."""

        self.__yaml_data = yaml_data
        self._parse_common()
        self._parse_parameters()
        self._parse_windows()

    def run(self, debug=False):
        """Runs the loaded session"""
        assert len(self._windows) > 0

        first = True
        for window in self._windows:
            window.create(first)
            if debug:
                window.debug()
            first = False

        tmux_wrapper = tmux.TmuxWrapper(self._server_name)
        if self._default_window:
            tmux_wrapper.tmux_call(
                [
                    "select-window",
                    "-t",
                    self._session_name + ":" + self._default_window,
                ]
            )

    def _parse_common(self):
        if self.__yaml_data is None:
            print("parse_common was called without yaml data loaded.")
            raise RuntimeError

        if "common" in self.__yaml_data and self.__yaml_data["common"]:
            common = self.__yaml_data["common"]
            if "before_commands" in common and common["before_commands"]:
                self._before_commands = common["before_commands"]

            if "default_window" in common and common["default_window"]:
                self._default_window = common["default_window"]

    def _parse_overwrites(self, data_string):
        """Separates a comma-separated list of foo=val1,bar=val2 into a dictionary."""
        if data_string is None:
            return None

        overwrites = dict()
        param_list = data_string.split(",")
        for param in param_list:
            key, value = param.split("=")
            overwrites[key] = value

        return overwrites

    def _parse_parameters(self):
        if self.__yaml_data is None:
            print("parse_parameters was called without yaml data loaded.")
            raise RuntimeError
        if "parameters" in self.__yaml_data:
            self._parameters = self.__yaml_data["parameters"]

        print("Parameters found in session config:")
        print(
            " - "
            + "\n - ".join(
                "{} = {}".format(key, value)
                for key, value in list(self._parameters.items())
            )
        )
        if self._runtime_params:
            print("Parameters found during runtime (overwrites):")
            print(
                " - "
                + "\n - ".join(
                    "{} = {}".format(key, value)
                    for key, value in list(self._runtime_params.items())
                )
            )
            # Overwrite parameters given from command line
            self._parameters.update(self._runtime_params)

        self._replace_parameters(self.__yaml_data)

    def _replace_parameters(self, data):
        if isinstance(data, dict):
            for key, value in list(data.items()):
                data[key] = self._replace_parameters(value)
        elif isinstance(data, list):
            for index, item in enumerate(data):
                data[index] = self._replace_parameters(item)
        elif isinstance(data, str):
            for key, value in list(self._parameters.items()):
                # print('-\nValue {}: {}\n='.format(value, type(data)))
                # if isinstance(value, str):
                # print('replacing {} in {}'.format(key, data))
                data = re.sub(r"\${%s}" % (key), str(value), data)
        return data

    def _parse_windows(self):
        if self.__yaml_data is None:
            print("parse_windows was called without yaml data loaded.")
            raise RuntimeError

        if "windows" in self.__yaml_data:
            if self.__yaml_data["windows"] is None:
                raise cme.InvalidConfig("The 'windows' block must not be empty.")
            for window in self.__yaml_data["windows"]:
                if "if" in window:
                    print("Detected if condition for window " + window["name"])
                    if window["if"] not in self._parameters:
                        print(
                            "Skipping window "
                            + window["name"]
                            + " because parameter "
                            + window["if"]
                            + " was not found."
                        )
                        continue
                    elif not check_boolean_field(self._parameters[window["if"]]):
                        print(
                            "Skipping window "
                            + window["name"]
                            + " because parameter "
                            + window["if"]
                            + " is switched off globally"
                        )
                        continue
                    else:
                        print(
                            "condition fulfilled: {} == {}".format(
                                window["if"], self._parameters[window["if"]]
                            )
                        )
                if "unless" in window:
                    print("Detected unless condition for window " + window["name"])

                    if window["unless"] in self._parameters:
                        if check_boolean_field(self._parameters[window["unless"]]):
                            print(
                                "Skipping window "
                                + window["name"]
                                + " because parameter "
                                + window["unless"]
                                + " is switched on globally"
                            )
                            continue
                        else:
                            print(
                                "condition fulfilled: {} == {}".format(
                                    window["unless"], self._parameters[window["unless"]]
                                )
                            )

                kwargs = dict()
                kwargs["before_commands"] = [cmd for cmd in self._before_commands]

                kwargs.update(window)

                print(kwargs)

                self._windows.append(
                    Window(self._server_name, self._session_name, **kwargs)
                )
        else:
            raise cme.InvalidConfig("No window section found in session config")



import subprocess
from unittest import mock

import yaml

import catmux.tmux_wrapper as tmw
from catmux.split import Split as Split
from catmux.session import Session


@mock.patch("subprocess.check_output")
def test_tmux_wrapper(mock_popen):
    server_name = "my_server"
    wrapper = tmw.TmuxWrapper(server_name)

    send_keys = "b"
    wrapper.send_keys(send_keys)
    mock_popen.assert_called_once_with(
        ["tmux", "-L", server_name, "send-keys", send_keys, "C-m"]
    )
    mock_popen.reset_mock()

    wrapper.send_keys(send_keys, target_window="foobar")
    mock_popen.assert_called_once_with(
        ["tmux", "-L", server_name, "send-keys", "-t", "foobar", send_keys, "C-m"]
    )
    mock_popen.reset_mock()

    wrapper.split(target_window="foobar")
    mock_popen.assert_called_once_with(
        ["tmux", "-L", server_name, "split-window", "-t", "foobar"]
    )
    mock_popen.reset_mock()


@mock.patch("subprocess.check_output")
def test_run_split(mock_popen):
    server_name = "my_server"

    split_data = {"commands": ["echo 'hello'", "echo 'world'"]}
    split = Split(**split_data)
    split.run(server_name)

    for cmd in split_data["commands"]:
        mock_popen.assert_any_call(["tmux", "-L", server_name, "send-keys", cmd, "C-m"])


@mock.patch("subprocess.check_output")
def test_full_circle(mock_popen):
    CONFIG = """common:
    before_commands:
        - echo "hello"
        - echo "world"
    default_window: foobar
parameters:
    replacement_param: schubidoo
    show_layouts: true

windows:
    - name: foo
      if: show_layouts
      commands:
        - echo "${replacement_param}"
    - name: bar
      layout: tiled
      delay: 1
      splits:
        - commands:
          - echo "first_split"
        - commands:
          - echo "second_split"
"""
    server_name = "my_server"
    session_name = "my_session"
    session = Session(server_name=server_name, session_name=session_name)
    session.init_from_yaml(yaml.safe_load(CONFIG))

    session.run(debug=True)
    calls = [
        ["rename-window", "-t", "my_session:$", "foo"],
        ["send-keys", "-t", "my_session:foo", 'echo "hello"', "C-m"],
        ["send-keys", "-t", "my_session:foo", 'echo "world"', "C-m"],
        ["send-keys", "-t", "my_session:foo", 'echo "schubidoo"', "C-m"],
        ["select-window", "-t", "my_session:foobar"],
        ["new-window", "-t", "my_session"],
        ["rename-window", "-t", "my_session:$", "bar"],
        ["send-keys", "-t", "my_session:bar", 'echo "first_split"', "C-m"],
        ["split-window", "-t", "my_session:bar"],
        ["send-keys", "-t", "my_session:bar", 'echo "second_split"', "C-m"],
        ["select-layout", "-t", "my_session:bar", "tiled"],
    ]
    for call in calls:
        mock_popen.assert_any_call(["tmux", "-L", server_name] + call)



# -- BEGIN LICENSE BLOCK ----------------------------------------------

# catmux
# Copyright (C) 2018  Felix Mauch
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# -- END LICENSE BLOCK ------------------------------------------------

"""A split in a tmux session"""

import catmux.tmux_wrapper as tmux


class Split(object):

    """A split is a pane where commands can be executed"""

    def __init__(self, **kwargs):
        """TODO: to be defined1."""

        if kwargs is not None:
            for (key, value) in kwargs.items():
                setattr(self, key, value)

    def debug(self, name="", prefix=""):
        """Prints all information about this window"""
        print(prefix + "- Split " + name + ":")
        if hasattr(self, "commands"):
            print(prefix + "  commands: ")
            print("\t- " + "\n\t- ".join(getattr(self, "commands")))

    def run(self, server_name, target_window=None):
        "Executes all configured commands" ""
        tmux_wrapper = tmux.TmuxWrapper(server_name=server_name)
        for command in getattr(self, "commands"):
            tmux_wrapper.send_keys(command, target_window)



# -- BEGIN LICENSE BLOCK ----------------------------------------------

# catmux
# Copyright (C) 2018  Felix Mauch
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# -- END LICENSE BLOCK ------------------------------------------------

"""Wraps usage of tmux commands"""
import subprocess


class TmuxWrapper:
    """Wrapper for tmux calls"""

    def __init__(self, server_name):
        self.server_name = server_name

    def send_keys(self, command, target_window=None):
        """Executes a command in the current tmux pane"""

        if target_window:
            self.tmux_call(["send-keys", "-t", target_window, command, "C-m"])
        else:
            self.tmux_call(["send-keys", command, "C-m"])

    def split(self, target_window):
        """Splits the current pane into two"""
        self.tmux_call(["split-window", "-t", target_window])

    def tmux_call(self, command_list):
        """Executes a tmux command"""
        tmux_cmd = ["tmux", "-L", self.server_name] + command_list

        # print(' '.join(tmux_cmd))
        _safe_call(tmux_cmd)


def _safe_call(cmd_list):
    """Makes a subprocess check_call and outputs a clear error message on failure and then exits"""
    try:
        subprocess.check_output(cmd_list)
        return True
    except subprocess.CalledProcessError as err_thrown:
        print('Error while calling "%s"', err_thrown.cmd)
        return False



import os
import sys

import pytest

from importlib import resources
import tempfile
import yaml

from catmux.session import Session
import catmux.exceptions


def test_common_block():
    CONFIG = """common:
    before_commands:
        - echo "hello"
        - echo "world"
    default_window: foobar
windows:
    - name: foo
      commands:
          - echo "bar"
"""

    session = Session("server", "name")
    session.init_from_yaml(yaml.safe_load(CONFIG))

    assert session._before_commands == ['echo "hello"', 'echo "world"']
    assert session._default_window == "foobar"


def test_no_common_block():
    CONFIG = """windows:
    - name: left-right
      splits:
        - commands:
          - echo "left"
    - name: foo
      splits:
        - commands:
          - echo "left"
"""
    session = Session("server", "name")
    session.init_from_yaml(yaml.safe_load(CONFIG))

    assert len(session._before_commands) == 0
    assert session._default_window == None


def test_empty_common_block():
    CONFIG = """common:
windows:
    - name: left-right
      splits:
        - commands:
          - echo "left"
    - name: foo
      splits:
        - commands:
          - echo "left"
"""
    session = Session("server", "name")
    session.init_from_yaml(yaml.safe_load(CONFIG))

    assert len(session._before_commands) == 0
    assert session._default_window == None


def test_empty_before_commands():
    CONFIG = """common:
    before_commands:
windows:
    - name: left-right
      splits:
        - commands:
          - echo "left"
    - name: foo
      splits:
        - commands:
          - echo "left"
"""
    session = Session("server", "name")
    session.init_from_yaml(yaml.safe_load(CONFIG))

    assert len(session._before_commands) == 0
    assert session._default_window == None


def test_missing_parameter_for_if_unless():
    CONFIG = """common:
    before_commands:
        - echo "hello"
        - echo "world"
    default_window: foobar
windows:
    - name: left-right
      if: show_layouts
      splits:
        - commands:
          - echo "left"
    - name: foo
      unless: show_layouts
      splits:
        - commands:
          - echo "left"
"""
    session = Session("server", "name")
    session.init_from_yaml(yaml.safe_load(CONFIG))


def test_parameter_replacement():
    CONFIG = """common:
    before_commands:
        - echo "hello"
        - echo "world"
    default_window: foobar
parameters:
    replacement_param: schubidoo
    show_layouts: true

windows:
    - name: foo
      if: show_layouts
      commands:
        - echo "${replacement_param}"
    - name: bar
      unless: show_layouts
      commands:
        - echo "I will not be printed when layouts are shown"
"""
    session = Session("server", "name")
    session.init_from_yaml(yaml.safe_load(CONFIG))

    assert session._parameters["replacement_param"] == "schubidoo"
    assert getattr(session._windows[0].splits[0], "commands")[0] == 'echo "schubidoo"'


def test_split_without_command():
    CONFIG = """windows:
    - name: foo
      layout: tiled
"""
    session = Session("server", "name")
    with pytest.raises(catmux.exceptions.InvalidConfig):
        session.init_from_yaml(yaml.safe_load(CONFIG))


def test_empty_config():
    CONFIG = """foo: bar
"""
    session = Session("server", "name")
    with pytest.raises(catmux.exceptions.InvalidConfig):
        session.init_from_yaml(yaml.safe_load(CONFIG))


def test_empty_windows_block():
    CONFIG = """windows:
"""
    session = Session("server", "name")
    with pytest.raises(catmux.exceptions.InvalidConfig):
        session.init_from_yaml(yaml.safe_load(CONFIG))


def test_parsing_overwrites():
    CONFIG = """---
common:
  before_commands:
    - echo "This is a catmux window :)"
  default_window: some_other_window
parameters:
  show_layouts: false
  replacement_param: world
windows:
    - name: foo
      commands:
        - echo "bar"
"""
    session = Session("server", "name", runtime_params=None)
    session.init_from_yaml(yaml.safe_load(CONFIG))

    session = Session(
        "server", "name", runtime_params="show_layouts=true, replacement_param=foobar"
    )
    session.init_from_yaml(yaml.safe_load(CONFIG))


def test_init_from_file():
    with resources.path(
        ".".join(["catmux", "resources"]), "example_session.yaml"
    ) as catmux_session:
        session = Session("server", "name")
        session.init_from_filepath(catmux_session)


def test_illegal_file():
    fd, session_config = tempfile.mkstemp()
    CONFIG = """[block name]
this is not a yaml file;
this is something else.
"""
    with os.fdopen(fd, "w") as f:
        f.write(CONFIG)

    session = Session("server", "name")
    with pytest.raises(catmux.exceptions.InvalidConfig):
        session.init_from_filepath(session_config)


def test_parsing_without_yaml():
    session = Session("server", "name")
    with pytest.raises(RuntimeError):
        session._parse_common()
    with pytest.raises(RuntimeError):
        session._parse_parameters()
    with pytest.raises(RuntimeError):
        session._parse_windows()



# -- BEGIN LICENSE BLOCK ----------------------------------------------

# catmux
# Copyright (C) 2018  Felix Mauch
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# -- END LICENSE BLOCK ------------------------------------------------

"""Contains the Window object"""
import time

import catmux.tmux_wrapper as tmux
from catmux.split import Split
from catmux.exceptions import InvalidConfig


class Window(object):

    """Class to represent a tmux window structure"""

    def __init__(self, server_name, session_name, **kwargs):
        """TODO: to be defined1."""

        self.server_name = server_name
        self.session_name = session_name

        split_list = kwargs.pop("splits", None)
        if not split_list:
            if "commands" in kwargs:
                split_dict = dict()
                split_dict["commands"] = kwargs.pop("commands")
                split_list = [split_dict]
            else:
                raise InvalidConfig(
                    f"No splits and no commands given for window '{kwargs['name']}'."
                )

        print(split_list)

        self.splits = list()
        for split_data in split_list:
            self.splits.append(Split(**split_data))

        if kwargs is not None:
            for (key, value) in kwargs.items():
                setattr(self, key, value)

    def debug(self):
        """Prints all information about this window"""
        print("\n----- {} -----".format(getattr(self, "name")))
        if hasattr(self, "before_commands"):
            print("before_commands: ")
            print("\t- " + "\n\t- ".join(getattr(self, "before_commands")))
        print("Splits:")
        for counter, split in enumerate(self.splits):
            split.debug(name=str(counter), prefix=" ")

    def create(self, first=False):
        """Creates the window"""
        tmux_wrapper = tmux.TmuxWrapper(server_name=self.server_name)
        target_window = ":".join([self.session_name, getattr(self, "name")])
        if not first:
            tmux_wrapper.tmux_call(["new-window", "-t", self.session_name])
        tmux_wrapper.tmux_call(
            ["rename-window", "-t", f"{self.session_name}:$", getattr(self, "name")]
        )
        for counter, split in enumerate(self.splits):
            if counter > 0:
                tmux_wrapper.split(target_window)

            if hasattr(self, "before_commands"):
                for cmd in getattr(self, "before_commands"):
                    tmux_wrapper.send_keys(cmd, target_window=target_window)
            split.run(server_name=self.server_name, target_window=target_window)

        if hasattr(self, "layout"):
            tmux_wrapper.tmux_call(
                ["select-layout", "-t", target_window, getattr(self, "layout")]
            )

        if hasattr(self, "delay"):
            print(
                "Window {} requested delay of {} seconds".format(
                    getattr(self, "name"), getattr(self, "delay")
                )
            )
            time.sleep(getattr(self, "delay"))



class InvalidConfig(Exception):
    """A given catmux config is not valid."""

    def __init__(self, message):
        self.message = message
        super().__init__("Illegal catmux config: " + self.message)



#!/usr/bin/env python

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="catmux",
    version="0.3.4",
    license="MIT",
    author="Felix Exner",
    author_email="felix_mauch@web.de",
    description="A tmux orchestration package.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fmauch/catmux",
    packages=setuptools.find_packages(),
    scripts=["script/catmux_create_session"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["pyyaml"],
    python_requires=">=3.7",
    package_data={"catmux.resources": ["*.yaml", "*.txt", "*.conf"]},
)



import yaml

import catmux.session

def test_debug_split():
    split_data = {"commands": ["echo 'hello'"]}
    split = catmux.split.Split(**split_data)
    split.debug (name="test_split", prefix="")

def test_debug():
    CONFIG = """common:
    before_commands:
        - echo "hello"
        - echo "world"
    default_window: foobar
windows:
    - name: left-right
      splits:
        - commands:
          - echo "left"
    - name: foo
      splits:
        - commands:
          - echo "left"
"""
    session = catmux.session.Session("server", "name")
    session.init_from_yaml(yaml.safe_load(CONFIG))
    session._windows[0].debug()




import os

from catmux.session import check_boolean_field
from catmux.tmux_wrapper import _safe_call as safe_call
from catmux.prefix import get_prefix
import catmux.resources


def test_boolean_field():
    assert check_boolean_field("yes") == True
    assert check_boolean_field("Yes") == True
    assert check_boolean_field("YES") == True
    assert check_boolean_field("true") == True
    assert check_boolean_field("True") == True
    assert check_boolean_field("TRUE") == True
    assert check_boolean_field("t") == True
    assert check_boolean_field("T") == True
    assert check_boolean_field("1") == True
    assert check_boolean_field("no") == False
    assert check_boolean_field("No") == False
    assert check_boolean_field("NO") == False
    assert check_boolean_field("false") == False
    assert check_boolean_field("False") == False
    assert check_boolean_field("FALSE") == False
    assert check_boolean_field(True) == True
    assert check_boolean_field(False) == False


def test_safe_call():
    assert safe_call(["bash", "-c", "exit 0"]) == True
    assert safe_call(["bash", "-c", "exit 1"]) == False
    assert safe_call(["bash", "-c", "return 0"]) == False  #  illegal bash


def test_get_prefix():
    prefix = os.path.join(get_prefix(), "__init__.py")
    assert prefix == catmux.resources.__file__
